#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 13/01/2003
#################################################################################
#
# Manage Gallery - Script part of Ethereal Realms, designed to manage galleries
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date handler
use Ethereal::Filter;						# Filtering handler
use Ethereal::Home;						# File handler
use Ethereal::Image;						# Image handling
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handlings
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table control
use Ethereal::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash
my $tmpl;							# Template handle

my $expire;							# Expire information
my $upload;							# Upload information

my %gal;							# Gallery handler

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect();


	# Prefetch
	$upload = $cgi->upload('UPLOAD_WORK');

	# Parameter handling
	$param    = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Fetch hash
	$database->GetHashGallery(\%gal);

	# Template handle
	$tmpl = new Ethereal::Template(\%gal);


	# Document header
	print $cgi->header();
	$database->DocumentGetGalleryHeader();


	# Active gallery component
	if ($gal{'SetActive'})
	{
		# Login handle
		$login    = new Ethereal::Login($database, $cgi, $param, \%gal);

		# Authenthication
		if ($login->GetVerificationNormal())
		{
			# Retreive expiry
			($expire) = $database->DataGetPuppeteerExpire($param->{'USER'});

			# Verified
			if ($expire > time)
			{
				# Work specific
				if (defined($param->{'WORK'}))	
				{
					# Editing
					if ((defined($param->{$gal{'TxtDetails'}}))
					 || (defined($param->{'EDIT_WORK'})))
					{
						# Detailed view
						WorkDetail($database, $cgi, $param, $tmpl, \%gal);
					}

					# Creation and management
					else
					{
						# Default behaviour
						WorkSummary($database, $cgi, $param, $tmpl, \%gal, $upload);
					}
				}

				# Gallery specific
				else
				{
					# Editing
					if ((defined($param->{'GALLERY'}))
					 && ((defined($param->{$gal{'TxtDetails'}}))
					 ||  (defined($param->{'EDIT_GALLERY'}))))
					{
						# Detailed view
						GalleryDetail($database, $cgi, $param, $tmpl, \%gal);
					}

					# Creation and management
					else
					{
						# Default behaviour
						GallerySummary($database, $cgi, $param, $tmpl, \%gal);
					}
				}
			}

			else
			{
				# Non verified
				$tmpl->Show('TmplMsg', MMESSAGE=>$gal{'MsgExpire'});
			}
		}
	}

	else
	{
		# Non-active
		$tmpl->Show('TmplMsg', MMESSAGE=>$gal{'MsgDisabled'});
	}

	# Document footer
	$database->DocumentGetGalleryFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Check Data
#
# This subroutine exists to verify information that is sent to it.   Basically
# this makes a more consistent interface and simplifies matters.

sub CheckData
{
	#####################
	# Data members
	my $choice   = shift;					# Choice of the matter
	my $value    = shift;					# Value to verify
	my $limit    = shift;					# Optional minimum limit


	#####################
	# Program area

	# Text field check
	if ($choice == 0)
	{
		# Allows A-Z and spaces
		return 1 if ($value =~ /^\w[\w\s\'\"\.\,\:\;\?\-]{$limit,}$/);
	}

	# Email check
	elsif ($choice == 1)
	{
		# Has the semblence of an Email address
		return 1 if ($value =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s);
	}


	# If all else fail
	return 0;
}

# Alias for E-Eail validity
sub CheckEMail
{ CheckData(1, shift); }

# Alias for Lenght
sub CheckString
{ CheckData(0, shift, shift); }


#####################
# Gallery Detail
#
# Gallery detail, will handle the finer points of a gallery itself.  This
# includes setting names, genres et cetera.

sub GalleryDetail
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash

	my $data     = $database->{'HANDLE'};			# Direct database handle
	my $statement;						# Database query statement

	my $date;						# Date handler
	my $filter;						# Filter handler
	my $option;						# Option handler
	my $table;						# Table creation

	my $descr;						# Description of types
	my $exist;						# Existence of value
	my $id;							# Gallery number
	my $name;						# Gallery name
	my $res;						# Results
	my $time;						# Current time

	my $url;						# Self referencing url	
	my $imain;						# Main inline parameters
	my $iwork;						# Work inline parameters

	my $comment;						# Comment options
	my $display;						# Display options
	my $genre;						# Genre options
	my $puppet;						# Puppet pen names
	my $rating;						# Rating options
	my $sort;						# Sorting options
	my $submt;						# Submit button
	my $type;						# Type options

	my @comment;						# Comment options array
	my @display;						# Display options array
	my @genre;						# Genre options array
	my @puppet;						# Puppet list 
	my @rating;						# Rating options array
	my @sort;						# Sorting options array
	my @type;						# Type options array


	#####################
	# Program area	

	# Initiate instances
	$date   = new Ethereal::Date();
	$option = new Ethereal::Option(); 
	$table  = new Ethereal::Table($cgi);


	# Retreive ID
	($id, $name) = split(/ - /, $param->{'GALLERY'});

	# Information modification
	if (defined($param->{'EDIT_GALLERY'}))
	{
		# Retreive time
		$time = time;

		# Initial type setup
		if (defined($param->{'GTYPE'}))
		{
			# Determine type
			$type = $option->GalleryType($gal->{'OptType'}, $param->{'GTYPE'});

			# Update database
			$data->do("UPDATE Gallery
				SET GalleryType=?,
				    GalleryUpdated=?
				WHERE GalleryID=?
				  AND PuppeteerLogin=?", {},

			 $type,
			 $time,
			 $id,
			 $param->{'USER'});

			# Cleanup
			$param->Cleanup('GTYPE');
		}

		# Standard
		else
		{
			# Initiate handle
			$filter = new Ethereal::Filter();

			# Existence
			($exist) = $database->DataGetGalleryID($param->{'GNAME'});


			# Main update
			# Description
			$param->{'GDESCR'} = $filter->StripHTML($param->{'GDESCR'});
			$param->{'GDESCR'} = substr(DBI::neat($param->{'GDESCR'}, 252), 1, -1);
			$param->{'GDESCR'} = (length($param->{'GDESCR'}) > 15) ? $param->{'GDESCR'} : undef;

			# Determin value
			$name    = (defined($exist)) ? $name : $param->{'GNAME'};

			# Options
			$comment = $option->GalleryComment($gal->{'OptComment'}, $param->{'GCOMMENT'});
			$sort    = $option->GallerySort($gal->{'OptSort'}, $param->{'GSORT'});


			# Update
			# Entry
			$data->do("UPDATE Gallery
				SET GalleryName=?,
				    GalleryRating=?,
				    GalleryGenre=?,
				    GalleryDescription=?,
				    GalleryPenName=?,
				    GalleryDisplay=?,
		 		    GalleryComments=?,
				    GallerySort=?,
				    GalleryUpdated=?
				WHERE GalleryID=?
				  AND PuppeteerLogin=?", {},

			 $name,
			 $param->{'GRATING'},
			 $param->{'GGENRE'},
			 $param->{'GDESCR'},
			 $param->{'GPUPPET'},
			 $param->{'GDISPLAY'},
			 $comment,
			 $sort,
			 $time,
			 $id,
			 $param->{'USER'});

			# Parameter
			$param->{'GALLERY'} = $id . ' - ' . $name;
		}
	}

	# Cleanup
	$param->Cleanup($gal->{'TxtDetails'});



	# Information gathering
	# Prepare and execute
	$statement = $data->prepare("SELECT
		 GalleryPenName,
		 GalleryName,
		 GalleryRating,
		 GalleryGenre,
		 GalleryDescription,
		 GalleryComments,
		 GallerySort,
		 GalleryType,
		 GalleryDisplay,
		 GalleryCreated,
		 GalleryUpdated
		FROM Gallery
		WHERE GalleryID=?
		  AND PuppeteerLogin=?");
	$statement->execute($id, $param->{'USER'});


	# Information display
	# Fetch results
	if ($res = $statement->fetchrow_hashref())
	{
		# Start Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), EDIT_GALLERY=>'True');

		# Verify initial access
		if ($res->{'GalleryUpdated'} < 1)
		{
			# Pull document
			($descr) = $database->DocumentGetGalleryType();

			# Pull options
			$option->Split(\@type, $gal->{'OptType'});


			# Generate widgets
			$submt = $cgi->submit($gal->{'TxtSelect'});
			$type  = $cgi->popup_menu('GTYPE', \@type, $type[0]);
			
			# Replace
			$descr =~ s/WGSUBMT/$submt/gs;
			$descr =~ s/WGTYPES/$type/gs;


			# Display document
			print $descr, "\n";
		}


		# Default behaviour
		else
		{
			# Back track
			$url   = $cgi->url(-relative=>1);
			$imain = $param->EmbedInline(
				USER=>$param->{'USER'}, 
				CRYPT=>$param->{'CRYPT'}
		 	 );
			$iwork = $param->EmbedInline(
				USER=>$param->{'USER'}, 
				CRYPT=>$param->{'CRYPT'},
				GID=>$id,
			 	WORK=>'True'	
			);

			# Genre
			$database->GetListGenre(\@genre, $res->{'GalleryType'});	

			# Gather information
			$option->Split(\@comment, $gal->{'OptComment'});
			$option->Split(\@display, $gal->{'OptDisplay'});
			$option->Split(\@rating,  $gal->{'OptRating'});
			$option->Split(\@sort,    $gal->{'OptSort'});

			# Puppet
			$database->GetListPuppetYours(\@puppet, $param->{'USER'});


			# Initial discovery
			$comment = $option->GalleryCommentRev($gal->{'OptComment'}, $res->{'GalleryComments'});
			$sort    = $option->GallerySortRev($gal->{'OptSort'}, $res->{'GallerySort'});

			# Generate widgets
			$puppet  = $cgi->popup_menu('GPUPPET',  \@puppet,  $res->{'GalleryPenName'});
			$genre   = $cgi->popup_menu('GGENRE',   \@genre,   $res->{'GalleryGenre'});
			$rating  = $cgi->popup_menu('GRATING',  \@rating,  $res->{'GalleryRating'});

			$comment = $cgi->radio_group('GCOMMENT', \@comment, $comment, 'true');
			$display = $cgi->radio_group('GDISPLAY', \@display, $res->{'GalleryDisplay'}, 'true');
			$sort    = $cgi->radio_group('GSORT',    \@sort, $sort, 'true');


			# Display information
			# Title
			$tmpl->ShowMain($gal->{'TagGalleryDetail'});

			# Link
			print "<CENTER>",
			      "<A HREF=\"$url$imain\">$gal->{'TagGalListing'}</A> ",
			      "<A HREF=\"$url$iwork\">$gal->{'TagGalWork'}</A> ",
			      "</CENTER><BR>";


			# Table top
			$table->MakeTop();

			# Key identifier	
			$table->MakeSingle($tmpl->PassTitle($res->{'GalleryName'}));	
			$table->MakeBlank();


			# Basic identifiers
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagGalName'}),
				$cgi->textfield('GNAME', $res->{'GalleryName'}, 25, 30)
			 );
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalGenre'}), $genre);
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalRating'}), $rating);
			$table->MakeBlank();


			# Alternate identity
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalPenName'}), $puppet);
			$table->MakeBlank();
			

			# Settings
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalComment'}), $comment);
			$table->MakeBlank();
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalSort'}),    $sort);
			$table->MakeBlank();
			$table->MakeValid($tmpl->PassSub($gal->{'TagGalDisplay'}), $display);
			$table->MakeBlank();


			# Description
			$table->MakeSingle($tmpl->PassTitle($gal->{'TagGalDescription'}));
			$table->MakeSingle($cgi->textarea(-name=>'GDESCR',-rows=>3,-columns=>75,-wrap=>'SOFT',-default=>$res->{'GalleryDescription'}));
			$table->MakeBlank();


			# Submission widgets
			$table->MakeValid($cgi->submit($gal->{'TxtUpdate'}), $cgi->reset($gal->{'TxtReset'}));

			# Table bottom
			$table->MakeBlank();
			$table->MakeSingle($tmpl->Pass('TmplStatus',
				MGALLERY => $id,
				MGALTYPE => ucfirst($res->{'GalleryType'}),
				MCREATED => $date->GetDateShort($res->{'GalleryCreated'}),
				MUPDATED => $date->GetDateShort($res->{'GalleryUpdated'}),
				MGALLINK => "$gal->{LnkGallery}/view/$id$imain")
			 );
			$table->MakeBottom();
		}

		# End of form
		print $cgi->end_form(), "\n";
	}


	# Finish query
	$statement->finish();
}


#####################
# Gallery Summary
#
# Gallery summary is used to display a list of all galleries for the
# user and allow them to modify them as need be.   Since gallery amounts
# should total less then puppets, the list will be a pop-up menu.

sub GallerySummary
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash

	my $data     = $database->{'HANDLE'};			# Direct database handle
	my $page;						# File handler
	my $table;						# Table creation

	my $buttons;						# Management buttons
	my $listing;						# List of galleries

	my $exist;						# Existence
	my $id;							# Last insert ID
	my $message;						# Notice message
	my $puppet;						# Default puppet name
	my $time;						# Timestamps

	my @galleries;						# Array of galleries


	#####################
	# Program area	

	# Information handling
	# Additions with names greater then 3 
	if ((defined($param->{$gal->{'TxtCreate'}}))
	 && (CheckString($param->{'ADD_GALLERY'}, 2)))
	{
		# Check for existence
		($exist)  = $database->DataGetGalleryID($param->{'ADD_GALLERY'});

		# Name not already taken
		unless (defined($exist))
		{
			# Retreive base information
			($puppet) = $database->DataGetDefault($param->{'USER'});

			# Time
			$time = time;


			# Creation
			$data->do("INSERT INTO Gallery
				(PuppeteerLogin,
				 GalleryName,
				 GalleryRating,
				 GalleryGenre,
				 GalleryDisplay,
				 GalleryPenName,
				 GalleryCreated)
				VALUES(?,?,?,?,?,?,?)", {},

			 $param->{'USER'},
			 $param->{'ADD_GALLERY'},
			 $gal->{'SetRating'},
			 $gal->{'SetGenre'},
			 $gal->{'SetDisplay'},
			 $puppet,
			 $time);


			# Determine ID
			($id) = $database->DataGet("SELECT LAST_INSERT_ID() FROM Gallery");


			# Remove if exists
			# Directory
			if (-d "$gal->{SetRoot}/$id")
			{
				# Create instance
				$page = new Ethereal::Home($database);

				# Remove directory
				$page->DirClear("$gal->{SetRoot}/$id");
			}

			# File
			if (-f "$gal->{SetRoot}/$id")
			{
				# Unlink
				unlink("$gal->{SetRoot}/$id");
			}


			# Attempt to create
			unless (mkdir("$gal->{SetRoot}/$id", 0775))
			{
				# Failed
				# Remove database entry
				$data->do("DELETE FROM Gallery
					WHERE GalleryID=?
					  AND PuppeteerLogin=?", {},
				 $id,
				 $param->{'USER'});

				# Signal notice
				$message = $gal->{'MsgError'} . $!;
			}

			else
			{
				# Message of success
				$message = $gal->{'MsgCreated'};
			}
		}

		# Notify
		else
		{
			# Existence message
			$message = $gal->{'MsgExists'};
		}

		# Cleanup
		$param->Cleanup($gal->{'TxtCreate'});
	}

	# Removal
	if ((defined($param->{$gal->{'TxtDelete'}}))
	 && (defined($param->{'CONFIRM'})))
	{
		# Retreive ID
		($id)    = split(/ - /, $param->{'GALLERY'});

		# Confirm
		($exist) = $database->DataGetGalleryName($id, $param->{'USER'});


		# Systematically remove
		if ((defined($id))
		 && (defined($exist)))
		{
			# Create instance
			$page = new Ethereal::Home($database);

			# Remove directory
			$page->DirClear("$gal->{SetRoot}/$id");


			# Remove database entry
			$data->do("DELETE FROM Gallery
				WHERE GalleryID=?
				  AND PuppeteerLogin=?", {},
			 $id,
			 $param->{'USER'});

			# Signal notice
			$message = $gal->{'MsgRemoved'};
		}

		# Cleanup
		$param->Cleanup($gal->{'TxtDelete'});
	}

	# Always cleanup
	$param->Cleanup('CONFIRM');


	# Information display
	# Initiate handle
	$table = new Ethereal::Table($cgi);


	# Retrieve list
	$database->GetList(\@galleries, "SELECT CONCAT(GalleryId, ' - ', GalleryName)
		 AS Gallery 
		 FROM Gallery 
		 WHERE PuppeteerLogin=?
		 ORDER BY GalleryName", $param->{'USER'});

	# Widget generation
	$listing = $cgi->scrolling_list('GALLERY', \@galleries, $galleries[0], 5);
	$buttons = $cgi->submit($gal->{'TxtDetails'}) . '<BR><BR>'
		.  $cgi->submit($gal->{'TxtDelete'})
		.  $cgi->checkbox(-name=>'CONFIRM', -label=>'');


	# Display
	# Title
	$tmpl->ShowMain($gal->{'TagGallery'});

	# Form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Table top
	$table->MakeTop();

	# Notice messages
	if (defined($message))
	{
		# Display through template
		$table->MakeSingle($tmpl->Pass('TmplNotice', MNOTICE=>$message));
		$table->MakeBlank();
	}


	# Gallery listing
	$table->MakeSingle($tmpl->PassSub($gal->{'TagSelectGallery'}));
	$table->MakeValid($listing, $buttons, 'middle');
	$table->MakeBlank();

	# Gallery creation
	$table->MakeSingle($tmpl->PassTitle($gal->{'TagAddGallery'}));
	$table->MakeValid(
		$tmpl->PassSub($gal->{'TagAddGalleryName'}),
		$cgi->textfield('ADD_GALLERY', '', 25, 30)
	 );
	$table->MakeSingle($cgi->submit($gal->{'TxtCreate'}));
	
	# Table Bottom
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}

#####################
# Work Detail
#
# Work detail, will handle the finer points of a works themselves.  This
# includes setting names, genres et cetera.

sub WorkDetail
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash

	my $data     = $database->{'HANDLE'};			# Direct database handle
	my $date;						# Date handler
	my $filter;						# Filter handler
	my $table;						# Table creation

	my $res;						# Query results
	my $statement;						# Database query statement

	my $gid;						# Gallery id
	my $wid;						# Work id

	my $url;						# Self referencing link
	my $imain;						# Main menu inline
	my $iwork;						# Works listing

	my $exist;						# Existence
	my $gtype;						# Gallery type
	my $time;						# Time of day
	my $rating;						# Item rating
	my $thumb;						# Thumbnail
	my $votes;						# Votes

	my %system;						# System hash


	#####################
	# Program area

	# Initiate instances
	$date  = new Ethereal::Date();
	$table = new Ethereal::Table($cgi);


	# Gallery type
	($gtype) = $database->DataGetGalleryType($param->{'GID'}, $param->{'USER'});

	# Primary set
	($wid) = $database->DataGetWorkID($param->{'WORK'}, $param->{'GID'});
	($gid) = $param->{'GID'}; 


	# Update information
	if (defined($param->{'EDIT_WORK'}))
	{
		# Common
		# Initiate handle
		$filter = new Ethereal::Filter();


		# Time
		$time = time;

		# Existence
		($exist) = $database->DataGetWorkID($param->{'WNAME'}, $param->{'GID'});


		# Main update
		# Description
		$param->{'WDESCR'} = $filter->StripHTML($param->{'WDESCR'});
		$param->{'WDESCR'} = substr(DBI::neat($param->{'WDESCR'}, 252), 1, -1);
		$param->{'WDESCR'} = (length($param->{'WDESCR'}) > 15) ? $param->{'WDESCR'} : undef;

		# Determin value
		$param->{'WNAME'}  = (defined($exist)) ? $param->{'WORK'} : $param->{'WNAME'};


		# Image
		if ($gtype eq 'image')
		{
			# Generate thumbnail
			$thumb = "$gal->{SetUrl}/$gid/$wid/thumb-work-$param->{WTHUMB}.jpg";

			# Update work
			$data->do("UPDATE Work
				SET WorkName=?,
				    WorkDescription=?,
				    WorkImageThumbnail=?,
				    WorkTimestamp=?
				WHERE WorkID=?
				  AND GalleryID=?", {},
			 $param->{'WNAME'},
			 $param->{'WDESCR'},
			 $thumb,
			 $time,
			 $wid,
			 $gid);
		}

		# Text
		else
		{
			# Preserve text
			my $preserve = (defined($param->{'WPRESERVE'})) ? 'yes' : 'no';

			# Safe HTML
			if ($preserve eq 'no')
			{
				# Initial filter
				$param->{'WTEXT'} = $filter->SafeHTML($param->{'WTEXT'});

				# Create hyperlinks
                		$param->{'WTEXT'} =~ s/(http:\/\/.*?)(\s|$)/<A HREF="$1" TARGET="_blank">$gal->{TagHyperlink}<\/A> /gm;
                		$param->{'WTEXT'} =~ s/([\w\.\-]*?\@[\w\.\-]*?\..*?)(\s|$)/<A HREF="mailto:$1">$gal->{TagEmail}<\/A> /gm;
			}


			# Update work
			$data->do("UPDATE Work
				SET WorkName=?,
				    WorkDescription=?,
				    WorkText=?,
				    WorkTextPreserve=?,
				    WorkTimestamp=?
				WHERE WorkID=?
				  AND GalleryID=?", {},
			 $param->{'WNAME'},
			 $param->{'WDESCR'},
			 $param->{'WTEXT'},
			 $preserve,
			 $time,
			 $wid,
			 $gid);
		}


		# General update
		$data->do("UPDATE Gallery
			SET GalleryUpdated=?
			WHERE GalleryID=?
			  AND PuppeteerLogin=?", {},

		 $time,
		 $param->{'GID'},
		 $param->{'USER'});
	}

	# Cleanup
	$param->Cleanup($gal->{'TxtDetails'}, 'WPRESERVE');


	# Information gathering
	# Prepare and execute
	$statement = $data->prepare("SELECT 
		  WorkName,
		  WorkDescription,
		  WorkImage,
		  WorkImageThumbnail,
		  WorkImageHeight,
		  WorkImageWidth,
		  WorkText,
		  WorkTextPreserve,
		  WorkTimestamp
		FROM Work
		WHERE WorkID=?
		  AND GalleryID=?");
	$statement->execute($wid, $gid);


	# Results successful
	if ($res = $statement->fetchrow_hashref())
	{
		# Calculate rating
		($rating, $votes) = $database->DataGetWorkAvg($gid, $wid);


		# Display information
		# Combined

		# Back track
		$url   = $cgi->url(-relative=>1);
		$imain = $param->EmbedInline(
			USER=>$param->{'USER'}, 
			CRYPT=>$param->{'CRYPT'}
 		 );
		$iwork = $param->EmbedInline(
			USER=>$param->{'USER'}, 
			CRYPT=>$param->{'CRYPT'},
			GID=>$gid,
		 	WORK=>'True'	
		);


		# Title
		$tmpl->ShowMain($gal->{'TagWorkDetail'});

		# Link
		print "<CENTER>",
		      "<A HREF=\"$url$imain\">$gal->{TagGalListing}</A> ",
		      "<A HREF=\"$url$iwork\">$gal->{TagGalWork}</A> ",
		      "</CENTER><BR>";


		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), EDIT_WORK=>'True', WORK=>$res->{'WorkName'});


		# Table top
		$table->MakeTop();
		$table->MakeSingle($tmpl->PassTitle($res->{'WorkName'}));
		$table->MakeBlank();


		# Work Name
		$table->MakeValid(
			$tmpl->PassSub($gal->{'TagWrkName'}),
			$cgi->textfield('WNAME', $res->{'WorkName'}, 25, 30)
		 );


		# Image specific
		if ($gtype eq 'image')
		{
			# Associate with hash
			$database->GetHashSystem(\%system);


			# Image height and width
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagWrkWidth'}),
				"<I>$res->{WorkImageWidth}</I>"
			 );	
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagWrkHeight'}),
				"<I>$res->{'WorkImageHeight'}</I>"
			 );	
			$table->MakeBlank();


			# Thumbnail Selection
			$table->MakeSingle($tmpl->PassTitle($gal->{'TagWrkThumbnail'}));


			# Display thumbnails
			my @thumb = qw(neg mid pos);

			# One at a time
			foreach $thumb (@thumb)
			{
				# Selected or not
				if ($res->{'WorkImageThumbnail'} =~ /$thumb/)
				{
					# Default
					$table->MakeValid(
						"<IMG SRC=\"$gal->{SetUrl}/$gid/$wid/thumb-work-$thumb.jpg\" HEIGHT=\"$system{'SetThumbnail'}\" WIDTH=\"$system{'SetThumbnail'}\">",
						"<CENTER><INPUT TYPE=\"radio\" NAME=\"WTHUMB\" VALUE=\"$thumb\" CHECKED></CENTER>",
						"middle"
					 );
				}
				else
				{
					# Non-default
					$table->MakeValid(
						"<IMG SRC=\"$gal->{SetUrl}/$gid/$wid/thumb-work-$thumb.jpg\" HEIGHT=\"$system{'SetThumbnail'}\" WIDTH=\"$system{'SetThumbnail'}\">",
						"<CENTER><INPUT TYPE=\"radio\" NAME=\"WTHUMB\" VALUE=\"$thumb\"></CENTER>",
						"middle"
					 );
				}

				# Spacer
				$table->MakeBlank();
			}
		}

		# Text specific
		else
		{
			# Generate widget
			my $preserve = ($res->{'WorkTextPreserve'} eq 'yes')
			  ? $cgi->checkbox(-name=>'WPRESERVE', -label=>'', -checked=>'on')
			  : $cgi->checkbox(-name=>'WPRESERVE', -label=>'');

			# Preserve text
			$table->MakeValid(
				$tmpl->PassSub($gal->{'TagWrkPreserve'}),
				$preserve);
			$table->MakeBlank();

			# Body of text
			$table->MakeSingle($tmpl->PassTitle($gal->{'TagWrkTxt'}));
			$table->MakeSingle($cgi->textarea(-name=>'WTEXT',-rows=>15,-columns=>75,-wrap=>'SOFT',-default=>$res->{'WorkText'}));
			$table->MakeBlank();
		}

		# Description
		$table->MakeSingle($tmpl->PassTitle($gal->{'TagWrkDescr'}));		
		$table->MakeSingle($cgi->textarea(-name=>'WDESCR',-rows=>3,-columns=>75,-wrap=>'SOFT',-default=>$res->{'WorkDescription'}));


		# Submission widgets
		$table->MakeValid($cgi->submit($gal->{'TxtUpdate'}), $cgi->reset($gal->{'TxtReset'}));
		$table->MakeBlank();

		# Table bottom
		$table->MakeSingle($tmpl->Pass('TmplRatings',
			MGALLERY => $gid,
			MITEMNUM => $wid,
			MOLVOTES => $votes,
			MRATINGS => $rating,
			MUPDATED => $date->GetDateShort($res->{'WorkTimestamp'}),
			MGALLINK => "$gal->{LnkGallery}/view/$gid$imain",
			MWRKLINK => "$gal->{LnkGallery}/view/$gid/$wid$imain")
		 );

		$table->MakeBottom();

		# End form
		print $cgi->end_form, "\n";
	}

	# End query
	$statement->finish();
}


#####################
# Work Summary
#
# Will list all of the works, while allowing the user to add and remove
# them as need be.  This will be similiar to the interface for galleries
# only work specific.

sub WorkSummary
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $gal      = shift;					# Gallery hash
	my $upload   = shift;					# Upload handle

	my $data     = $database->{'HANDLE'};			# Direct database handle
	my $image;						# Image library
	my $page;						# File handler
	my $table;						# Table creation

	my $buttons;						# Management buttons
	my $listing;						# List of works
	my $message;						# Notice message
	my $time;						# Current time

	my $gid;						# Gallery ID
	my $gtype;						# Gallery type
	my $wid;						# Work ID
	
	my $url;						# Self referencing link
	my $imain;						# Inline main menu

	my $contents;						# File contents
	my $err;						# Write error message
	my $ext;						# File extension
	my $mime;						# Mime type

	my @works;						# Array of works


	#####################
	# Program area	

	# Gallery type
	($gtype) = $database->DataGetGalleryType($param->{'GID'}, $param->{'USER'});


	# Information handling
	# Additions with names greater then 3 
	if ((defined($param->{$gal->{'TxtCreate'}}))
	 && (CheckString($param->{'ADD_WORK'}, 2)))
	{
		# Duplicate check
		($wid)  = $database->DataGetWorkID($param->{'ADD_WORK'}, $param->{'GID'});
		($time) = time;

		# Safe to proceed
		unless (defined($wid))
		{
			# Database work
			# Create database entry
			$data->do("INSERT INTO Work
				(GalleryID,
				 WorkName,
				 WorkTimestamp)
				VALUES (?,?,?)", {},

			 $param->{'GID'},
			 $param->{'ADD_WORK'},
			 $time);

			# Update gallery
			$data->do("UPDATE Gallery
				SET GalleryUpdated=?
				WHERE GalleryID=?
				  AND PuppeteerLogin=?", {},

			 $time,
			 $param->{'GID'},
			 $param->{'USER'});


			# Retreive Work and Gallery ID
			($wid) = $database->DataGet("SELECT LAST_INSERT_ID() FROM Work");
			($gid) = $param->{'GID'};


			# File work
			# Image galleries only
			if ($gtype eq 'image')
			{
				# Create instances
				$page  = new Ethereal::Home($database, "$gal->{SetRoot}/$gid/$wid");
				$image = new Ethereal::Image($database, "$gal->{SetRoot}/$gid/$wid");


				# Stale entries
				# Remove file
				unlink $page->{'BASEDIR'} if (-f $page->{'BASEDIR'});

				# Remove directory
				$page->DirClear($page->{'BASEDIR'}) if (-d $page->{'BASEDIR'});
			

				# Attempt to create
				# Failed to create
				unless (mkdir($page->{'BASEDIR'}, 0775))
				{
					# Failed
					# Remove database entry
					$data->do("DELETE FROM Work
						WHERE GalleryID=?
						  AND WorkID=?", {},
					 $gid,
					 $wid);

					# Signal notice
					$message = $gal->{'MsgError'} . $!;
				}

				# Created directory
				else
				{
					# Upload file
					# Valid handle
					if (defined($upload))
					{
						# Determin MIME type
						$mime = $cgi->uploadInfo($param->{'UPLOAD_WORK'})->{'Content-Type'};

						# Determin file name
						$ext = $param->{'UPLOAD_WORK'};
						$ext =~ /^.*\.(\w+)$/;
						$ext = $1;

						# Valid MIME
						if ($mime =~ /^image/i)
						{
							# Retreive image
							while (<$upload>) { $contents = "$contents$_"; }

							# Write image
							$err = $page->FileWrite("work.$ext", $contents);


							# Successful
							if ($err == 1)
							{
								# Create thumbnails
								$image->Thumbnail("work.$ext");

								# Retreive sizes
								my ($width, $height, $size) = $image->Size("work.$ext");


								# Update database
								$data->do("UPDATE Work
									SET WorkImage=?,
									    WorkImageThumbnail=?,
									    WorkImageHeight=?,
									    WorkImageWidth=?,
									    WorkImageSize=?
									WHERE WorkID=?
									  AND GalleryID=?", {},

								 "$gal->{SetUrl}/$gid/$wid/work.$ext",
								 "$gal->{SetUrl}/$gid/$wid/thumb-work-mid.$ext",
								 $height,
								 $width,
								 $size,
								 $wid,
								 $gid);


								# Message of success
								$message = $gal->{'MsgCreated'};
							}

							else
							{
								# Unable to write
								$message = $gal->{'MsgError'} . $err;
							}
	
						}
						else
						{
							# Improper MIME type
							$message = $gal->{'MsgMime'};
						}
					}
				}
			}
			else
			{
				# Nothing to create
				# Message of success
				$message = $gal->{'MsgCreated'};
			}
		}
		else
		{
			# Name exists
			$message = $gal->{'MsgExists'};
		}

		# Cleanup
		$param->Cleanup($gal->{'TxtCreate'});
	}

	# Removal
	if ((defined($param->{$gal->{'TxtDelete'}}))
	 && (defined($param->{'CONFIRM'})))
	{
		# Determine Work/Gallery ID
		($wid)  = $database->DataGetWorkID($param->{'WORK'}, $param->{'GID'});
		($gid)  = $param->{'GID'};
		($time) = time;

		# Remove
		if (defined($wid))
		{
			# Database entry
			$data->do("DELETE FROM Work
				WHERE WorkID=?
				  AND GalleryID=?", {}, 

			 $wid, 
			 $gid);	

			# Update gallery
			$data->do("UPDATE Gallery
				SET GalleryUpdated=?
				WHERE GalleryID=?
				  AND PuppeteerLogin=?", {},

			 $time,
			 $gid,
			 $param->{'USER'});


			# Directory structure
			if ($gtype eq 'image')
			{
				# Create instances
				$page  = new Ethereal::Home($database, "$gal->{SetRoot}/$gid/$wid");

				# Remove directory
				$page->DirClear($page->{'BASEDIR'}) if (-d $page->{'BASEDIR'});
			}

			# Define message
			$message = $gal->{'MsgRemoved'};
		}

		# Cleanup
		$param->Cleanup($gal->{'TxtDelete'});
	}

	# Always cleanup
	$param->Cleanup('CONFIRM');

	# Back track
	$url   = $cgi->url(-relative=>1);
	$imain = $param->EmbedInline(
		USER=>$param->{'USER'}, 
		CRYPT=>$param->{'CRYPT'}
 	 );


	# Information display
	# Initiate handle
	$table = new Ethereal::Table($cgi);


	# Retrieve list
	$database->GetList(\@works, "SELECT WorkName
		 FROM Work 
		 WHERE GalleryID=?
		 ORDER BY WorkName", $param->{'GID'});

	# Widget generation
	$listing = $cgi->scrolling_list('WORK', \@works, $works[0], 10);
	$buttons = $cgi->submit($gal->{'TxtDetails'}) . '<BR><BR>'
		.  $cgi->submit($gal->{'TxtDelete'})
		.  $cgi->checkbox(-name=>'CONFIRM', -label=>'');


	# Display
	# Title
	$tmpl->ShowMain($gal->{'TagWork'});

	# Link
	print "<CENTER><A HREF=\"$url$imain\">$gal->{'TagGalListing'}</A></CENTER><BR>";


	# Form
	# Multipart allows for uplaods
	if ($gtype eq 'image')
	{ print $cgi->start_multipart_form(); }

	# Decreased abuse (standard)
	else
	{ print $cgi->start_form(); }
	print $param->EmbedNormal($param->Flat()), "\n";


	# Table top
	$table->MakeTop();

	# Notice messages
	if (defined($message))
	{
		# Display through template
		$table->MakeSingle($tmpl->Pass('TmplNotice', MNOTICE=>$message));
		$table->MakeBlank();
	}


	# Gallery listing
	$table->MakeSingle($tmpl->PassSub($gal->{'TagSelectWork'}));
	$table->MakeValid($listing, $buttons, 'middle');
	$table->MakeBlank();

	# Gallery creation
	# Base
	$table->MakeSingle($tmpl->PassTitle($gal->{'TagAddWork'}));
	$table->MakeValid(
		$tmpl->PassSub($gal->{'TagAddWorkName'}),
		$cgi->textfield('ADD_WORK', '', 25, 30)
	 );

	# Upload field
	if ($gtype eq 'image')
	{
		$table->MakeValid(
			$tmpl->PassSub($gal->{'TagAddWorkFile'}),
			$cgi->filefield('UPLOAD_WORK', '', 25, 140)
		 );
	}

	# Submit widget
	$table->MakeSingle($cgi->submit($gal->{'TxtCreate'}));
	
	# Table Bottom
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}
